@@ -22,8 +22,8 @@ main(int argc, char *argv[]) {
 	set_progname(argv[0]);
 
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+	textdomain(PACKAGE_NAME);
 
 	if (argc == 2 && !strcmp(argv[1], "-V"))
 		print_version_and_exit();
@@ -35,7 +35,7 @@ main(int argc, char *argv[]) {
 		}
 	}
 
-	fd = getfd();
+	fd = getfd(NULL);
 
 	if (argc == 1) {
 		/* deallocate all unused consoles */
