@@ -2,19 +2,6 @@
  * dumpkeys.c
  *
  * derived from version 0.81 - aeb@cwi.nl
- * Fix: escape quotes and backslashes in strings
- * Fix: after  dumpkeys > x; loadkeys x; dumpkeys > y
- *      the files x and y should be identical
- * Added: compose key support
- *
- * for 0.83: output a "+" for KT_LETTER
- * for 0.85: with -i option: also output MAX_DIACR
- * for 0.86: with -l option: also tell about synonyms
- * for 0.87: output "charset iso-8859-x" so that loadkeys
- *      can handle the output of dumpkeys -c
- * for 0.88: handle sparse keymaps
- * for 0.94: support alt_is_meta
- * for 0.96: option -1
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,8 +15,10 @@
 #include <string.h>
 #include <errno.h>
 #include "ksyms.h"
+#include "getfd.h"
 #include "modifiers.h"
 #include "nls.h"
+#include "version.h"
 
 #ifndef KT_LETTER
 #define KT_LETTER KT_LATIN
@@ -39,17 +28,30 @@
 #define MAX_NR_KEYMAPS NR_KEYMAPS
 #endif
 
-#define VERSION "0.98"
-char *progname;
-extern int getfd();
 static int fd;
-
 static int verbose;
+static int nr_keys = 0;			/* probably 112, 128, 256 or 512 */
 
-int keymap_index[MAX_NR_KEYMAPS];		/* inverse of good_keymap */
+int keymap_index[MAX_NR_KEYMAPS];	/* inverse of good_keymap */
 int good_keymap[MAX_NR_KEYMAPS], keymapnr, allocct;
 
-void get_keymaps(void) {
+/* note: asking for n > 255 is not meaningful: ke.kb_index is uchar */
+static int
+has_key(int n) {
+	struct kbentry ke;
+
+	ke.kb_table = 0;	/* plain map is always present */
+	ke.kb_index = n;
+	return !ioctl(fd, KDGKBENT, (unsigned long)&ke);
+}
+
+static void
+find_nr_keys(void) {
+	nr_keys = (has_key(255) ? 256 : has_key(127) ? 128 : 112);
+}
+
+static void
+get_keymaps(void) {
 	int i, j;
 	struct kbentry ke;
 
@@ -60,7 +62,8 @@ void get_keymaps(void) {
 	    j = ioctl(fd, KDGKBENT, (unsigned long)&ke);
 	    if (j && errno != EINVAL) {
 		perror("KDGKBENT");
-		fprintf(stderr, _("KDGKBENT error at index 0 in table %d: "), i);
+		fprintf(stderr,
+			_("KDGKBENT error at index 0 in table %d\n"), i);
 		exit(1);
 	    }
 	    if (!j && ke.kb_value != K_NOSUCHMAP) {
@@ -78,12 +81,14 @@ void get_keymaps(void) {
 	}
 	if (good_keymap[0] != 0) {
 	    fprintf(stderr,
-		    _("%s: plain map not allocated? very strange ...\n"), progname);
+		    _("%s: plain map not allocated? very strange ...\n"),
+		    progname);
 	    /* this is not fatal */
 	}
 }
 
-void print_keymaps(void) {
+static void
+print_keymaps(void) {
 	int i,m0,m;
 
 	printf("keymaps ");
@@ -101,47 +106,60 @@ void print_keymaps(void) {
 	printf("\n");
 }
 
-int get_bind(u_char index, u_char table) {
+static int
+get_bind(u_char index, u_char table) {
 	struct kbentry ke;
 
 	ke.kb_index = index;
 	ke.kb_table = table;
 	if (ioctl(fd, KDGKBENT, (unsigned long)&ke)) {
-		perror("KDGKBENT");
-		fprintf(stderr, _("KDGKBENT error at index %d in table %d: "),
-			index, table);
-		exit(1);
+		if (index < 128) {
+			perror("KDGKBENT");
+			fprintf(stderr, _("KDGKBENT error at index %d in table %d\n"),
+				index, table);
+			exit(1);
+		} else
+			return -1;
 	}
 	return ke.kb_value;
 }
 
-void print_keysym(int code, char numeric) {
+static void
+print_keysym(int code, char numeric) {
 	int t;
 	int v;
-	char *p;
+	const char *p;
+	int plus;
 
 	printf(" ");
 	t = KTYP(code);
 	v = KVAL(code);
-	if (t > KT_LETTER) {
-		printf("U+%04x          ", code ^ 0xf000);
+	if (t >= syms_size) {
+		code = code ^ 0xf000;
+		if (!numeric && (p = unicodetoksym(code)) != NULL)
+			printf("%-16s", p);
+		else
+			printf("U+%04x          ", code);
 		return;
 	}
+	plus = 0;
 	if (t == KT_LETTER) {
 		t = KT_LATIN;
 		printf("+");
+		plus++;
 	}
 	if (!numeric && t < syms_size && v < syms[t].size &&
 	    (p = syms[t].table[v])[0])
-		printf("%-16s", p);
+		printf("%-*s", 16 - plus, p);
 	else if (!numeric && t == KT_META && v < 128 && v < syms[0].size &&
 		 (p = syms[0].table[v])[0])
 		printf("Meta_%-11s", p);
 	else
-		printf("0x%04x          ", code);
+		printf("0x%04x         %s", code, plus ? "" : " ");
 }
 
-char valid_type(int t) {
+static char
+valid_type(int t) {
 	struct kbentry ke;
 	char status;
 
@@ -152,7 +170,8 @@ char valid_type(int t) {
 	return status;
 }
 
-u_char maximum_val(int t) {
+static u_char
+maximum_val(int t) {
 	struct kbentry ke, ke0;
 	int i;
 
@@ -180,7 +199,7 @@ int maxval[NR_TYPES];
 /* isgraph() does not know about iso-8859; printing the character
    unescaped makes the output easier to check. Maybe this should
    be an option. Use locale? */
-void
+static void
 outchar (unsigned char c) {
 	printf("'");
 	printf((c == '\'' || c == '\\') ? "\\%c"
@@ -189,9 +208,10 @@ outchar (unsigned char c) {
 	printf("'");
 }
 
-struct kbdiacrs kd;
+static struct kbdiacrs kd;
 
-void get_diacs(void) {
+static void
+get_diacs(void) {
 	static int got_diacs = 0;
 
 	if(!got_diacs && ioctl(fd, KDGKBDIACR, (unsigned long)&kd)) {
@@ -201,12 +221,14 @@ void get_diacs(void) {
 	got_diacs = 1;
 }
 
-int nr_of_diacs(void) {
+static int
+nr_of_diacs(void) {
 	get_diacs();
 	return kd.kb_cnt;
 }
 
-void dump_diacs(void) {
+static void
+dump_diacs(void) {
 	int i;
 
 	get_diacs();
@@ -222,11 +244,12 @@ void dump_diacs(void) {
 }
 #endif        
 
-void show_short_info(void) {
+static void
+show_short_info(void) {
 	int i;
 
 	printf(_("keycode range supported by kernel:           1 - %d\n"),
-	       NR_KEYS - 1);
+	       nr_keys - 1);
 	printf(_("max number of actions bindable to a key:         %d\n"),
 	       MAX_NR_KEYMAPS);
 	get_keymaps();
@@ -248,7 +271,7 @@ void show_short_info(void) {
 	       nr_of_diacs());
 }
 
-struct {
+static struct {
     char *name;
     int bit;
 } modifiers[] = {
@@ -259,15 +282,18 @@ struct {
     { "shiftl",	KG_SHIFTL },
     { "shiftr",	KG_SHIFTR },
     { "ctrll",	KG_CTRLL  },
-    { "ctrlr",	KG_CTRLR  }
+    { "ctrlr",	KG_CTRLR  },
+    { "capsshift",	KG_CAPSSHIFT  }
 };
 
-void dump_symbols(void) {
+static void
+dump_symbols(void) {
 	int t;
 	int v;
-	char *p;
+	const char *p;
 
-	printf(_("Symbols recognized by %s:\n(numeric value, symbol)\n\n"), progname);
+	printf(_("Symbols recognized by %s:\n(numeric value, symbol)\n\n"),
+	       progname);
 	for (t = 0; t < syms_size; t++) {
 	    if (syms[t].size) {
 		for (v = 0; v < syms[t].size; v++)
@@ -288,7 +314,8 @@ void dump_symbols(void) {
 	  printf("%s\t\t%3d\n", modifiers[t].name, 1 << modifiers[t].bit);
 }
 
-void print_mod(int x) {
+static void
+print_mod(int x) {
 	int t;
 
 	if (!x)
@@ -299,7 +326,8 @@ void print_mod(int x) {
 	    printf("%s\t", modifiers[t].name);
 }
 
-void print_bind(int bufj, int i, int j, char numeric) {
+static void
+print_bind(int bufj, int i, int j, char numeric) {
 	if(j)
 	    printf("\t");
 	print_mod(j);
@@ -313,7 +341,8 @@ void print_bind(int bufj, int i, int j, char numeric) {
 #define SEPARATE_LINES	2	/* one line for each (modifier,keycode) pair */
 #define	UNTIL_HOLE	3	/* one line for each keycode, until 1st hole */
 
-void dump_keys(char table_shape, char numeric) {
+static void
+dump_keys(char table_shape, char numeric) {
 	int i, j, k;
 	int buf[MAX_NR_KEYMAPS];
 	int isletter, islatin, isasexpected;
@@ -333,10 +362,12 @@ void dump_keys(char table_shape, char numeric) {
 	for (j = 0; j < MAX_NR_KEYMAPS; j++) {
 	     int ja = (j | M_ALT);
 	     if (j != ja && keymap_index[j] >= 0 && keymap_index[ja] >= 0)
-		  for (i = 1; i < NR_KEYS; i++) {
+		  for (i = 1; i < nr_keys; i++) {
 		       int buf0, buf1, type;
 
 		       buf0 = get_bind(i, j);
+		       if (buf0 == -1)
+			   break;
 		       type = KTYP(buf0);
 		       if ((type == KT_LATIN || type == KT_LETTER)
 			   && KVAL(buf0) < 128) {
@@ -359,9 +390,11 @@ void dump_keys(char table_shape, char numeric) {
 not_alt_is_meta:
 
 no_shorthands:
-	for (i = 1; i < NR_KEYS; i++) {
+	for (i = 1; i < nr_keys; i++) {
 	    for (j = 0; j < keymapnr; j++)
 	      buf[j] = get_bind(i, good_keymap[j]);
+	    if (buf[0] == -1)
+		break;
 
 	    if (table_shape == FULL_TABLE) {
 		printf("keycode %3d =", i);
@@ -463,10 +496,11 @@ no_shorthands:
 	}
 }
 
-void dump_funcs(void) {
+static void
+dump_funcs(void) {
 	int i;
 	struct kbsentry fbuf;
-	char *p;
+	unsigned char *p;
 
 	for (i = 0; i < MAX_NR_FUNC; i++) {
 		fbuf.kb_func = i;
@@ -492,35 +526,36 @@ void dump_funcs(void) {
 	}
 }
 
-/* bah - xgettext forces us to uglify this */
-void usage(void) {
-	fprintf(stderr, _("dumpkeys version %s"), VERSION);
-	fprintf(stderr, _(
-"\n"
-"usage: dumpkeys [options...]\n"
-"\n"
-"valid options are:\n"
-"\n"
-"	-h --help	    display this help text\n"
-"	-i --short-info	    display information about keyboard driver\n"
-"	-l --long-info	    display above and symbols known to loadkeys\n"
-"	-n --numeric	    display keytable in hexadecimal notation\n"
-"	-f --full-table	    don't use short-hand notations, one row per keycode\n"
-"	-1 --separate-lines one line per (modifier,keycode) pair\n"
-"	   --funcs-only	    display only the function key strings\n"
-"	   --keys-only	    display only key bindings\n"
-"	   --compose-only   display only compose key combinations\n"
-"	-c --charset=iso-8859-{1,2,3,4,5,7,8,9}\n"
-"	             koi8-{r,u}		\n"
-"			    interpret character action codes to be from the\n"
-"			    specified character set\n"
-));
+static void
+usage(void) {
+	fprintf(stderr, _("dumpkeys version %s"), PACKAGE_VERSION);
+	fprintf(stderr, _("\
+\n\
+usage: dumpkeys [options...]\n\
+\n\
+valid options are:\n\
+\n\
+	-h --help	    display this help text\n\
+	-i --short-info	    display information about keyboard driver\n\
+	-l --long-info	    display above and symbols known to loadkeys\n\
+	-n --numeric	    display keytable in hexadecimal notation\n\
+	-f --full-table	    don't use short-hand notations, one row per keycode\n\
+	-1 --separate-lines one line per (modifier,keycode) pair\n\
+	   --funcs-only	    display only the function key strings\n\
+	   --keys-only	    display only key bindings\n\
+	   --compose-only   display only compose key combinations\n\
+	-c --charset="));
+	list_charsets(stderr);
+	fprintf(stderr, _("\
+			    interpret character action codes to be from the\n\
+			    specified character set\n\
+"));
 	exit(1);
 }
 
 int
 main (int argc, char *argv[]) {
-	const char *short_opts = "hilvsnf1S:c:";
+	const char *short_opts = "hilvsnf1S:c:V";
 	const struct option long_opts[] = {
 		{ "help",	no_argument,		NULL, 'h' },
 		{ "short-info",	no_argument,		NULL, 'i' },
@@ -532,8 +567,9 @@ main (int argc, char *argv[]) {
 		{ "funcs-only",	no_argument,		NULL, 't' },
 		{ "keys-only",	no_argument,		NULL, 'k' },
 		{ "compose-only",no_argument,		NULL, 'd' },
-		{ "verbose",	no_argument,		NULL, 'v' },
 		{ "charset",	required_argument,	NULL, 'c' },
+		{ "verbose",	no_argument,		NULL, 'v' },
+		{ "version",	no_argument,		NULL, 'V' },
 		{ NULL,	0, NULL, 0 }
 	};
 	int c;
@@ -545,11 +581,11 @@ main (int argc, char *argv[]) {
 	char keys_only = 0;
 	char diac_only = 0;
 
-	progname = argv[0];
+	set_progname(argv[0]);
 
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+	textdomain(PACKAGE_NAME);
 
 	while ((c = getopt_long(argc, argv,
 		short_opts, long_opts, NULL)) != -1) {
@@ -590,6 +626,8 @@ main (int argc, char *argv[]) {
 					usage();
 				printf("charset \"%s\"\n", optarg);
 				break;
+			case 'V':
+				print_version_and_exit();
 			case 'h':
 			case '?':
 				usage();
@@ -599,7 +637,9 @@ main (int argc, char *argv[]) {
 	if (optind < argc)
 		usage();
 
-	fd = getfd();
+	fd = getfd(NULL);
+
+	find_nr_keys();
 
 	if (short_info || long_info) {
 		show_short_info();
